module splines
contains
  ! natural spline
  subroutine spline(x,y,n,y2)
    implicit none
    integer n,i,k
    real(kind(1.d0)),intent(in):: x(n),y(n)
    real(kind(1.d0)),intent(out):: y2(n)
    real(kind(1.d0))::p,sig,u(n) ! automatic
    u(1)=0.d0
    do i=2,n-1
       sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
       p=sig*y2(i-1)+2.d0
       y2(i)=(sig-1.d0)/p
       u(i)=(6.d0*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))/(x(i) &
            -x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
    end do
    y2(n)=0.d0    
    do k=n-1,1,-1
       y2(k)=y2(k)*y2(k+1)+u(k)
    end do
  end subroutine spline
  subroutine splint(xa,ya,y2a,n,x,y)
    integer,intent(in):: n
    real(kind(1.d0)),intent(in):: x,xa(n),y2a(n),ya(n)
    real(kind(1.d0)),intent(out):: y
    integer:: k,khi,klo
    real(kind(1.d0)):: a,b,h
    klo=1
    khi=n
    do while(khi-klo > 1) 
       k=(khi+klo)/2
       if(xa(k) > x)then
          khi=k
       else
          klo=k
       endif
    end do
    h=xa(khi)-xa(klo)
    if (h == 0.d0) stop 'h=0, bad xa input in splint'
    a=(xa(khi)-x)/h
    b=(x-xa(klo))/h
    y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.d0
  end subroutine splint
end module splines
